#!/bin/bash
# Copyright Atomicorp, 2023
# All Rights Reserved

###############
# Globals
###############
VERSION="2.0.0"
MIN_VER="ClamAV 0.103.1"
DEBUG=0

###########################
# Logging
###########################
LOG_FILE="/var/ossec/logs/atomicorp-api.log"


###########################
#functions
###########################
function log_event () {
    MSG=$1
    echo "`date "+%b %d %H:%M:%S"` ${HOSTNAME} atomicorp-api-module-clamav: ${MSG}" >> ${LOG_FILE}
	logger -t atomicorp-api-module-clamav "${MSG} Agent ID: ${ID} "
}

function dist_detection() {
	if [ -f /etc/redhat-release ]; then
		RELEASE=/etc/redhat-release
	elif [ -f /etc/os-release ]; then
		RELEASE=/etc/os-release
	elif [ -f /etc/openvz-release ]; then
		RELEASE=/etc/openvz-release
	elif [ -f /etc/SuSE-release ]; then
		RELEASE=/etc/SuSE-release
	elif [ -f /etc/os-release ]; then
		RELEASE=/etc/os-release
	elif [ -f /etc/lsb-release ]; then
		RELEASE=/etc/lsb-release
	elif [ -f /etc/debian_version ]; then
		RELEASE=/etc/debian_version
	elif [ -f /etc/openvz-release ]; then
		RELEASE=/etc/openvz-release
	elif [ -f /etc/virtuozzo-release ]; then
		RELEASE=/etc/virtuozzo-release
	  	RELEASE=
	elif [[ $OSTYPE == "aix"* ]]; then
	  	PKG=aix
	fi

	PKG=rpm
	if [[ $OSTYPE == "aix"* ]]; then
	  	PKG=aix
	elif egrep -q "(release 5)" $RELEASE ; then
	  	DIST="el5"
	elif egrep -q "(release 6|release 2012)" $RELEASE ; then
	  	DIST="el6"
	elif egrep -q "(release 7|release 2014)" $RELEASE ; then
	  	DIST="el7"
	elif egrep -q "(release 8)" $RELEASE ; then
	  	DIST="el8"
	elif egrep -q "(release 9)" $RELEASE ; then
	  	DIST="el9"
	elif egrep -q "Red Hat Enterprise Linux.* 7" $RELEASE ; then
	  	DIST="el7"
	elif egrep -q "Red Hat Enterprise Linux.* 8" $RELEASE ; then
	  	DIST="el8"
	elif egrep -q "(Amazon Linux 2)" $RELEASE; then
	  	DIST="amazon"
	elif egrep -q "(Amazon Linux AMI|Amazon)" $RELEASE ; then
	  	DIST="el6"
	elif egrep -q "wheezy" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "jessie" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "stretch" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "buster" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "bookworm" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "lucid" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "precise" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "Raring Ringtail" $RELEASE ; then
	  	DIST="debian"
	  	PKG=deb
	elif egrep -q "Trusty Tahr" $RELEASE ; then
	  	DIST="ubuntu"
	  	PKG=deb
	elif egrep -q "Xenial" $RELEASE ; then
	  	DIST="ubuntu"
	  	PKG=deb
	elif egrep -q "Bionic" $RELEASE ; then
		DIST="ubuntu"
		PKG=deb
	elif egrep -q "Focal" $RELEASE ; then
		DIST="ubuntu"
		PKG=deb
	elif egrep -q "Jammy" $RELEASE ; then
		DIST="ubuntu"
		PKG=deb
	fi

}

clamd_settings() {
	if [[ "$PKG" == "rpm" ]]; then
		CLAM_CONF="/etc/clamd.conf"
		systemctl daemon-reload
		systemctl enable clamonacc
		systemctl enable clamav-daemon
	elif [[ "$PKG" == "deb" ]]; then
		CLAM_CONF="/etc/clamav/clamd.conf"
		systemctl daemon-reload
		systemctl enable clamav-clamonacc
		systemctl enable clamav-daemon
	fi
	
}

clamd_restart() {
	if [[ "$PKG" == "rpm" ]]; then
		systemctl restart clamav-daemon
		systemctl restart clamonacc
	elif [[ "$PKG" == "deb" ]]; then
		systemctl restart clamav-daemon
		systemctl restart clamav-clamonacc
	fi
}

clamd_version() {
	# standard
	if  [ -f /usr/sbin/clamd ]; then
		CLAMD_DAEMON=/usr/sbin/clamd
	elif [ -f /opt/freeware/bin/clamd ]; then
		CLAMD_DAEMON=/opt/freeware/bin/clamd
	else
		echo "Error: clamd not detected"
		exit 1
	fi

	C_VERSION=$(${CLAMD_DAEMON} --version |awk -F/ '{print $1}')
	if [[ "$C_VERSION" < "$MIN_VER" ]]; then
		echo "ERROR: Minimum version $MIN_VER required. $C_VERSION detected"
		echo 
		exit 1
	fi

}

dist_detection
clamd_version
clamd_settings


ID=$(sed '3q;d' /var/ossec/queue/ossec/.agent_info)
if [ -f /var/ossec/etc/shared/${ID}.malware-detection ] && [ -f $CLAM_CONF ]; then
	if [[ $DEBUG -ge 1 ]]; then
		log_event "Updating clamav settings: START"
	fi
    sed -i "s/^User.*/User root/g" $CLAM_CONF
    sed -i "s/^LocalSocketGroup.*//g" $CLAM_CONF
    sed -i "s/^LocalSocketMode.*//g" $CLAM_CONF
    sed -i "s/^OnAccessExcludeRootUID.*//g" $CLAM_CONF
    sed -i "s/^OnAccessExcludeUID.*//g" $CLAM_CONF
    sed -i "s/^OnAccessIncludePath.*//g" $CLAM_CONF
    sed -i "s/^OnAccessExcludePath.*//g" $CLAM_CONF
    sed -i "s/^OnAccessPrevention.*//g" $CLAM_CONF

	for line in $(cat /var/ossec/etc/shared/${ID}.malware-detection ); do

		if  [[ ${line} == uid* ]]; then
			NAME=$(echo $line |awk -F, '{ print $2}')
			if id -u ${NAME} >/dev/null 2>&1 ; then
				CLAM_UID=$(id -u ${NAME})
                if [[ ${CLAM_UID} -eq 0 ]]; then
					echo "OnAccessExcludeRootUID yes"  >> $CLAM_CONF
                else
					echo "OnAccessExcludeUID ${CLAM_UID}" >> $CLAM_CONF
                fi
            fi
		elif [[ ${line} == include* ]]; then
            CLAM_INCLUDE=$(echo $line |awk -F, '{ print $2}')
			if [ -d $CLAM_INCLUDE ]; then
               	echo "OnAccessIncludePath  $CLAM_INCLUDE" >> $CLAM_CONF
			fi
		elif [[ ${line} == exclude* ]]; then
            CLAM_EXCLUDE=$(echo $line |awk -F, '{ print $2}')
            echo "OnAccessExcludePath $CLAM_EXCLUDE" >> $CLAM_CONF
		elif [[ ${line} == setting* ]]; then
			CLAM_SETTING=$(echo $line |awk -F, '{ print $2}')
			CLAM_TOKEN=$(echo $line |awk -F, '{ print $3}')
			
			if grep -q "^${CLAM_SETTING}" $CLAM_CONF; then
				sed -i "s/^${CLAM_SETTING}.*/${CLAM_SETTING} ${CLAM_TOKEN}/g" $CLAM_CONF
			else
				echo "${CLAM_SETTING} ${CLAM_TOKEN}" >> $CLAM_CONF
			fi
		fi

	done
    sed -i '/^$/d' $CLAM_CONF
	clamd_restart
	if [[ $DEBUG -ge 1 ]]; then
		echo "DEBUG: clamav settings updated"
	fi
	
fi

